/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2016-2017, Josef Hahn

 This file is part of Clove.

 Clove is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Clove is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Clove.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/

/*!
* \class clove
* \brief Clove root namespace.
*/        

/*!
* \class clove.utils
* \brief Namespace for some utilities.
*/        

/*!
* \class clove.symbols
* \brief Namespace for symbol constants. 
* 
* They make a few Unicode symbols accessible by a name.
*/        

/*!
* \fn main_parts_resized()
* \memberof clove::utils
* \brief Call this method if resizing is required due to external situations.
* 
* It is not required to call this method in typical situations. A situation
* which requires this method is changing the stylesheets from outside.
* \param reason Optional arbitrary object which describes the reason for resizing. Can be evaluated by handlers in a custom way.
*/        

/*!
* \fn applyDefaultsToConfig()
* \memberof clove::utils
* \brief Returns a copy of `config` with the additional members from `defaults`, whereever
* there was no value in `config`.
* \param config The source object.
* \param defaults Default values for members to apply to the result whereever they are missing.
*/        

/*!
* \fn addOnDestroyHandler()
* \memberof clove::utils
* \brief Listen for the removal of a node in the dom tree and eventually execute a handler function.
* \param domnode The node in the dom tree to observe.
* \param handler The function to execute on removal of `domnode`.
*/        

/*!
* \fn arraysum()
* \memberof clove::utils
* \brief Computes the sum of all elements in an Array.
* \param x The Array to sum up.
*/        

/*!
* \fn findWidgetForDomNode()
* \memberof clove::utils
* \brief Finds the (most inner) clove::Widget a given dom node contains to.
* \param node The dom node.
*/        

/*!
* \fn getExtraSize()
* \memberof clove::utils
* \brief Computes the extra size of a dom node.
* 
* This is the difference between inner size and the outer size (i.e. borders, padding, ...).
* \param node The dom node to measure.
* \param withMargin If to include margins.
*/        

/*!
* \fn getInnerSize()
* \memberof clove::utils
* \brief Computes the inner size of a dom node.
* 
* This is the size without borders, padding, ...
* \param node The dom node to measure.
*/        

/*!
* \fn getOuterSize()
* \memberof clove::utils
* \brief Computes the outer size of a dom node.
* 
* This is the size including borders, padding, ...
* \param node The dom node to measure.
* \param withMargin If to include margins.
*/        

/*!
* \fn suspendResizing()
* \memberof clove::utils
* \brief Temporarily suspends all user interface resizing.
* 
* This is useful for performance reasons during intensive changes
* to the user interface.
* 
* Use this with caution and always call clove::utils::resumeResizing afterwards.
* \returns A token object used for resuming later on.
*/        

/*!
* \fn resumeResizing()
* \memberof clove::utils
* \brief Resumes the user interface resizing after it was suspended with clove::utils::suspendResizing.
* 
* It automatically takes care of resizing everything which diverged meanwhile.
* \param token The token object.
*/        

/*!
* \fn build()
* \memberof clove
* \brief Constructs new widgets according to a widget configuration and controlled by the build configuration.
* 
* Read the Manual for details.
* \param config The widget configuration; like a blueprint for the new widget.
* \param buildconfig The build configuration; specifies some aspects about the construction.
*/        

/*!
* \fn getByName()
* \memberof clove
* \brief Finds a widget by name in the clove root namescope.
* 
* Read the Manual for details.
* \param name The widget name.
*/        

/*!
* \fn cssLengthToPixels()
* \memberof clove::utils
* \brief Translates a css length specification string to a numeric pixel value.
* 
* Note: This does not make sense for all kinds of lengths, e.g. percentages will not work.
* \param v The css length.
* \param axis The axis ('x' or 'y').
*/        

/*!
* \fn insertToArray()
* \memberof clove::utils
* \brief Inserts a value into an array.
* \param array The array.
* \param i The position.
* \param v The new value to insert.
*/        

/*!
* \fn popup()
* \memberof clove::utils
* \brief Shows a popup.
* 
* This can show an arbitrary widget decoupled from the main user interface (swimming above it).
* 
* This call retuns a dialog result structure, containing the following:
* - `widget`: The popup widget as clove::Widget.
* - `namescope`: The root namescope for the popup as clove::NameScope.
* \param config The widget configuration, as for clove::build(), which specifies the popup.
* \param showconfig A configuration object which specifies some presentation aspects.
*/        

/*!
* \fn isDescendantOf()
* \memberof clove::utils
* \brief Checks if `d` is a descendant of `p` in the html dom node.
* \param d An html dom node.
* \param p An html dom node.
*/        

/*!
* \fn arrayToDatasource()
* \memberof clove::utils
* \brief Builds a clove::Datasource containing the same content as the input array.
* \param array The input array.
*/        

/*!
* \fn connectDatasourceToWidget()
* \memberof clove::utils
* \brief Connects a clove::Datasource to an internal widget interface (also removes the old connection).
* 
* This method is solely used by the inner implementation of a widget for using a datasource. It is not useful to call it in any other situation!
* \param widget A clove::Widget.
* \param datasource A clove::Datasource.
* \param dspropname The property name for storing the current datasource in the widget.
* \param rootptrpropname The property name to get the root clove::DatasourceValuePointer from the widget.
* \param inserthandlername The name of the widget's insert handler method.
* \param removehandlername The name of the widget's remove handler method.
* \param updatehandlername The name of the widget's update handler method.
* \param afterdisconnectedfct Optional function called just after disconnection of the old datasource.
* \param beforeconnectedfct Optional function called just before connecting the new datasource.
* \param afterconnectedfct Optional function called just after connecting the new datasource.
*/        

/*!
* \fn animateNode()
* \memberof clove::utils
* \brief Plays a css animation on a html dom node for one iteration and fires a callback afterwards.
* \param node The html dom node to animate.
* \param animationname The css animation name.
* \param animationduration The animation duration in milliseconds.
* \param animationendedfct a callback `function()` called when the animation ends.
*/        

/*!
* \fn setStyleClassAssigned()
* \memberof clove::utils
* \brief Sets or unsets a css class to an html dom node.
* \param node The html dom node to modify.
* \param clss The css class name.
* \param assigned If to assign or unassign it.
*/        

/*!
* \fn connectDatasourceToWidget()
* \memberof clove::utils
* \brief Connects a datasource to a widget.
* 
* Used internally in widget implementations.
* \param widget The clove::Widget.
* \param datasource The clove::Datasource to connect.
* \param dspropname The name of the datasource property.
* \param rootptrpropname The name of the root value pointer property.
* \param inserthandlername The name of the insert handler.
* \param removehandlername The name of the remove handler.
* \param updatehandlername The name of the update handler.
* \param afterdisconnectedfct This function is called after the last datasource is disconnected.
* \param beforeconnectedfct This function is called just before the datasource is connected.
* \param afterconnectedfct This function is called right after the datasource is connected.
*/        

/*!
* \fn populateUI()
* \memberof clove
* \brief Utility for user interface creation.
* 
* Using this function for building the main user interface (or other complex pieces) enhances the
* performance and can provide a loading indicator visible to the user.
* \param initfct A function which actually builds the user interface.
* \param populateconfig A configuration object which specifies some behavior aspects during initialization.
*/        

/*!
* \fn conversationDialog()
* \memberof clove
* \brief Shows a conversation dialog (small and easy dialog box).
* 
* This builds a small clove::Dialog with an arbitrary body widget and a button bar.
* It returns a dialog result structure, as clove::utils::popup(), also containing 
* `OnProceed`. This clove::Event triggers when the user clicks on one of the buttons in the button bar.
* 
* config may contain:
* - `title`: Dialog title text.
* - `body`: Inner widget as widget configuration like for clove::build().
* - `icon`: Icon as clove::Icon.
* - `buttons`: List of button label strings.
* \param config A conversation dialog configuration object.
*/        

/*!
* \fn messageDialog()
* \memberof clove
* \brief Shows a message dialog.
* 
* This builds a small clove::Dialog which contains a message and asks the user to click on a button.
* It returns an clove::Event which is triggered when the user clicked on a button.
* 
* config may contain:
* - `title`: Dialog title text.
* - `message`: The message text.
* - `buttons`: List of button label strings.
* 
* Note: In very exotic situations, it might be useful to access the complete dialog result structure 
* as clove::conversationDialog() would return. This is possible with the `dlgres` property
* of the returned object.
* \param config A conversation dialog configuration object.
*/        

/*!
* \fn inputDialog()
* \memberof clove
* \brief Shows an input dialog.
* 
* This builds a small clove::Dialog which asks the user for some text input.
* It returns an clove::Event which is triggered when the user finished.
* 
* config may contain:
* - `title`: Dialog title text.
* - `question`: The question text.
* - `buttons`: List of button label strings.
* - `defaultanswer`: The initial answer string.
* 
* Note: In very exotic situations, it might be useful to access the complete dialog result structure 
* as clove::conversationDialog() would return. This is possible with the `dlgres` property
* of the returned object.
* \param config A conversation dialog configuration object.
*/        

/*!
* \fn selectionDialog()
* \memberof clove
* \brief Shows a dialog with a list of choices.
* 
* This builds a small clove::Dialog which asks the user to choose an item from a list.
* It returns an clove::Event which is triggered when the user finished.
* 
* config may contain:
* - `title`: Dialog title text.
* - `question`: The question text.
* - `buttons`: List of button label strings.
* - `defaultanswer`: The initial answer string.
* - `choices`: The list of choice strings.
* 
* Note: In very exotic situations, it might be useful to access the complete dialog result structure 
* as clove::conversationDialog() would return. This is possible with the `dlgres` property
* of the returned object.
* \param config A conversation dialog configuration object.
*/        

/*!
* \property Visible
* \memberof clove
* \brief Visibility value for a visible widget.
*/        

/*!
* \property Invisible
* \memberof clove
* \brief Visibility value for a widget, which is invisible but occupies its room.
*/        

/*!
* \property InvisibleCollapsed
* \memberof clove
* \brief Visibility value for a widget, which is not visible and does not take any space.
*/        

/*!
* \property i18n
* \memberof clove
* \brief Provides internationalization features.
* 
* Read the Manual for details.
*/        

/*!
* \property rootNameScope
* \memberof clove
* \brief The Clove root namescope.
*/        

/*!
* \property OnMainResized
* \memberof clove::utils
* \brief Triggered when the main view resized.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property browser
* \memberof clove
* \brief Some data helpful for browser detection.
*/        

/*!
* \class clove::I18N
* \brief Internationalization support class.
* 
* Always use the instance clove.i18n. Read the Manual for details.
*/        

/*!
* \fn addString()
* \memberof clove::I18N
* \brief Adds a text to the internationalization text table.
* 
* The keys are ISO 639-1 language codes.
* 
* Example: `addString('PleaseWait', {'de':'Bitte warten', 'en':'Please wait'})`
* \param id The text identifier name, like 'FooBar'.
* \param texts A configuration object which holds a text representation for all target languages.
*/        

/*!
* \fn static parseLangCode()
* \memberof clove::I18N
* \brief Returns a clove string representation.
* 
* Only indended to be used by the infrastructure.
* \param lang A language code string as understood by addString().
*/        

/*!
* \class clove::EventArgs
* \brief Arguments for a particular incidence of a clove::Event.
* 
* It carries some additional informations about it. The details depend on the
* event (and the component which triggers it).
*/        

/*!
* \fn EventArgs()
* \memberof clove::EventArgs
* \param params Additional information about the event incidence.
*/        

/*!
* \fn skipExecution()
* \memberof clove::EventArgs
* \brief Marks the further event handling for skipping.
*/        

/*!
* \class clove::Event
* \brief An event. It can have some handlers (or: listeners) and can be triggered.
* 
* Read the Manual for details about the Clove event mechanism.
*/        

/*!
* \fn Event()
* \memberof clove::Event
*/        

/*!
* \fn trigger()
* \memberof clove::Event
* \brief Triggers the event.
* 
* This calls all registered handlers.
* \param params Additional information about the event incidence.
*/        

/*!
* \fn addHandler()
* \memberof clove::Event
* \brief Adds a handler (or: listener) to this event.
* 
* Whenever the event is triggered afterwards, the new handler is called.
* \param fct A handler function.
* \param owner Optional owner widget. If given, removeHandler will be called automatically after widget removal.
*/        

/*!
* \fn hasHandlers()
* \memberof clove::Event
* \brief Checks if this event has any handlers (otherwise triggering it doesn't have an effect).
*/       

/*!
* \fn addHandlerOnce()
* \memberof clove::Event
* \brief Like addHandler(), but automatically removes the handler after the event occurred once.
* \param fct A handler function.
* \param owner Optional owner widget. If given, removeHandler will be called automatically after widget removal.
*/        

/*!
* \fn removeHandler()
* \memberof clove::Event
* \brief Removes a handler from this event.
* \param fct The handler function to remove.
*/        

/*!
* \class clove::NameScope
* \brief A mixin realizing a new namescope.
* 
* Read the Manual for details.
*/        

/*!
* \fn getByName()
* \memberof clove::NameScope
* \brief Finds a widget by name within this namescope.
* \param name The widget name.
*/        

/*!
* \fn addChildNameScope()
* \memberof clove::NameScope
* \brief Adds a namescope to the children of this one, so a searcher will also search in this child.
* \param childnamescope The child namescope.
*/        

/*!
* \class clove::RootNameScope
* \extends clove::NameScope
* \brief A standalone namescope.
*/        

/*!
* \class clove::DatasourceValuePointer
* \brief A pointer to one node in a clove::Datasource.
*/        

/*!
* \fn DatasourceValuePointer()
* \memberof clove::DatasourceValuePointer
* Typically called only inside a clove::Datasource implementation.
* From outside, you should use clove::Datasource::valuePointer.
* \param irow The row index.
* \param icol The column index.
* \param backendObject The backend object. This depends on the datasource implementation.
* \param datasource The owning clove::Datasource.
*/        

/*!
* \property irow
* \memberof clove::DatasourceValuePointer
* \brief The row index.
*/        

/*!
* \property icol
* \memberof clove::DatasourceValuePointer
* \brief The column index.
*/        

/*!
* \property datasource
* \memberof clove::DatasourceValuePointer
* \brief The owning clove::Datasource.
*/        

/*!
* \property backendObject
* \memberof clove::DatasourceValuePointer
* \brief The backend object.
* 
* This depends on the datasource implementation.
*/        

/*!
* \fn parent()
* \memberof clove::DatasourceValuePointer
* \brief Returns the parent clove::DatasourceValuePointer.
*/ 

/*!
* \fn sibling()
* \memberof clove::DatasourceValuePointer
* \brief Returns a sibling clove::DatasourceValuePointer.
* \param irow The row index.
* \param icol The column index.
*/ 

/*!
* \fn rowCount()
* \memberof clove::DatasourceValuePointer
* \brief Returns the number of rows in this node.
*/ 

/*!
* \fn columnCount()
* \memberof clove::DatasourceValuePointer
* \brief Returns the number of columns in this node.
*/ 

/*!
* \fn getValue()
* \memberof clove::DatasourceValuePointer
* \brief Returns the value stored behind this pointer.
*/ 

/*!
* \fn static equals()
* \memberof clove::DatasourceValuePointer
* \brief Checks if two value pointers point to the same place.
* \param ptr1 The first clove::DatasourceValuePointer.
* \param ptr2 The second clove::DatasourceValuePointer.
*/ 

/*!
* \fn static toString()
* \memberof clove::DatasourceValuePointer
* \brief Returns a string representation for a value pointer.
* \param ptr The clove::DatasourceValuePointer.
*/        

/*!
* \class clove::Datasource
* \brief Base class for datasources.
* 
* Read the Manual for details.
*/        

/*!
* \fn getValue()
* \pure
* \memberof clove::Datasource
* \brief Returns the value for a given node.
* \param ptr The clove::DatasourceValuePointer.
*/        

/*!
* \fn getMetadata()
* \pure
* \memberof clove::Datasource
* \brief Returns an object of metadata information (like icons, status infos used for styling, ...). Optional.
* \param ptr The clove::DatasourceValuePointer.
*/        

/*!
* \fn changeValue()
* \pure
* \memberof clove::Datasource
* \brief Change the value for a given node.
* 
* This method is called from external places, e.g. when a user makes changes in a
* datasource-connected widget.
* \param ptr The clove::DatasourceValuePointer.
* \param value The new value.
*/        

/*!
* \fn rowCount()
* \pure
* \memberof clove::Datasource
* \brief Returns the number of rows for a given node.
* \param parent The parent as clove::DatasourceValuePointer.
*/        

/*!
* \fn columnCount()
* \pure
* \memberof clove::Datasource
* \brief Returns the number of columns for a given node.
* \param parent The parent as clove::DatasourceValuePointer.
*/        

/*!
* \fn valuePointer()
* \pure
* \memberof clove::Datasource
* \brief Constructs and returns a clove::DatasourceValuePointer for a given node.
* \param irow The row index.
* \param icol The column index.
* \param parent The parent as clove::DatasourceValuePointer.
*/        

/*!
* \fn parent()
* \pure
* \memberof clove::Datasource
* \brief Returns the parent node for a given node.
* \param ptr A node as clove::DatasourceValuePointer.
*/        

/*!
* \fn valuePointerNavigateInDepth()
* \memberof clove::Datasource
* \brief Returns a clove::DatasourceValuePointer resulting in the original one by navigation in depth.
* \param ptr A node as clove::DatasourceValuePointer.
* \param direction The direction (+1 or -1).
* \param mayexpandfct A `function(ptr)` which returns `true` iff this node's children are to be traversed.
*/        

/*!
* \property OnDataInsert
* \memberof clove::Datasource
* \brief Triggered when a node insertion takes place.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnDataRemove
* \memberof clove::Datasource
* \brief Triggered when a node removal takes place.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnDataUpdate
* \memberof clove::Datasource
* \brief Triggered when a node data update takes place.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \class clove::Headersource
* \brief Base class for headersources.
* 
* A headersource provides information for headers in data views.
* Read the Manual for details.
*/        

/*!
* \fn getRowHeader()
* \pure
* \memberof clove::Headersource
* \brief Returns the header configuration for a given row.
* \param irow The row index.
* \param parent The parent node as clove::DatasourceValuePointer.
*/        

/*!
* \fn getColumnHeader()
* \pure
* \memberof clove::Headersource
* \brief Returns the header configuration for a given column.
* \param irow The column index.
* \param parent The parent node as clove::DatasourceValuePointer.
*/        

/*!
* \fn rowHeadersVisible()
* \pure
* \memberof clove::Headersource
* \brief If row headers are visible.
* \param parent The parent node as clove::DatasourceValuePointer.
*/        

/*!
* \fn columnHeadersVisible()
* \pure
* \memberof clove::Headersource
* \brief If column headers are visible.
* \param parent The parent node as clove::DatasourceValuePointer.
*/        

/*!
* \property OnHeaderDataInsert
* \memberof clove::Headersource
* \brief Triggered when a new row or column of header data was inserted.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnHeaderDataRemove
* \memberof clove::Headersource
* \brief Triggered when a row or column of header data was removed.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnHeaderDataUpdate
* \memberof clove::Headersource
* \brief Triggered when header data were updated.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnHeaderVisibilityUpdated
* \memberof clove::Headersource
* \brief Triggered when header visibilities changed.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \class clove::NativeDatasourceValue
* \brief A node value implementation for clove::NativeDatasource.
* 
* Not intended for direct construction!
*/        

/*!
* \fn rowCount()
* \memberof clove::NativeDatasourceValue
* \brief Returns the number of children rows.
*/        

/*!
* \fn columnCount()
* \memberof clove::NativeDatasourceValue
* \brief Returns the number of children columns.
*/        

/*!
* \fn toRow()
* \memberof clove::NativeDatasourceValue
* \brief Returns the clove::NativeDatasourceValue which is in the same column as this one, but in a different row.
* \param ir The row index.
*/        

/*!
* \fn toColumn()
* \memberof clove::NativeDatasourceValue
* \brief Returns the clove::NativeDatasourceValue which is in the same row as this one, but in a different column.
* \param ic The column index.
*/        

/*!
* \fn valuePointer()
* \memberof clove::NativeDatasourceValue
* \brief Returns a clove::DatasourceValuePointer pointing to this node.
*/        

/*!
* \fn getValue()
* \memberof clove::NativeDatasourceValue
* \brief Returns the node value.
*/        

/*!
* \fn getMetadata()
* \memberof clove::NativeDatasourceValue
* \brief Returns the node metadata.
* 
* See also clove::Datasource::getMetadata().
*/        

/*!
* \fn setValue()
* \memberof clove::NativeDatasourceValue
* \brief Sets the node value.
* \param v The new value.
*/        

/*!
* \fn setMetadata()
* \memberof clove::NativeDatasourceValue
* \brief Sets a node metadata value.
* \param k The metadata key, like 'icon'.
* \param v The new value.
*/        

/*!
* \fn removeRow()
* \memberof clove::NativeDatasourceValue
* \brief Removes a row in this node.
* \param ir The row index.
*/        

/*!
* \fn removeColumn()
* \memberof clove::NativeDatasourceValue
* \brief Removes a column in this node.
* \param ic The column index.
*/        

/*!
* \fn addRow()
* \memberof clove::NativeDatasourceValue
* \brief Adds a row to this node.
* \param vals A list of new values (one for each column).
*/        

/*!
* \fn addColumn()
* \memberof clove::NativeDatasourceValue
* \brief Adds a column to this node.
* \param vals A list of new values (one for each row).
*/        

/*!
* \fn insertRow()
* \memberof clove::NativeDatasourceValue
* \brief Inserts a row to this node.
* \param i The insertion position.
* \param vals A list of new values (one for each column).
*/        

/*!
* \fn insertColumn()
* \memberof clove::NativeDatasourceValue
* \brief Inserts a column to this node.
* \param i The insertion position.
* \param vals A list of new values (one for each row).
*/  

/*!
* \fn getChild()
* \memberof clove::NativeDatasourceValue
* \brief Returns a child clove::NativeDatasourceValue.
* \param irow The row index.
* \param icol The column index.
*/  

/*!
* \class clove::NativeDatasource
* \extends clove::Datasource
* \extends clove::Headersource
* \brief A clove::Datasource implementation which stores all data in-memory.
*
* It provides an interface for populating it with data and can directly be constructed and used.
* It also implements clove::Headersource, so it can also carry row and column header configurations.
*/        

/*!
* \fn NativeDatasource()
* \memberof clove::NativeDatasource
* The configuration object allows the following keys:
* 
* - `readonly`: If this datasource may be changed from outside.
* - `rowHeadersVisible`: If row headers shall be visible in presentation.
* - `columnHeadersVisible`: If column headers shall be visible in presentation.
* \param config A configuration object.
*/        

/*!
* \fn root()
* \memberof clove::NativeDatasource
* \brief The root node as clove::NativeDatasourceValue.
*/  

/*!
* \fn valuePointerToNativeNode()
* \memberof clove::NativeDatasource
* \brief Returns the clove::NativeDatasourceValue for a node.
* \param ptr The clove::DatasourceValuePointer.
*/        

/*!
* \fn insertRow()
* \memberof clove::NativeDatasource
* \brief Inserts a new empty row.
* \param ir The row index.
* \param parent The parent node as clove::DatasourceValuePointer.
*/        

/*!
* \fn insertColumn()
* \memberof clove::NativeDatasource
* \brief Inserts a new empty column.
* \param ic The column index.
* \param parent The parent node as clove::DatasourceValuePointer.
*/        

/*!
* \fn removeRow()
* \memberof clove::NativeDatasource
* \brief Removes a row.
* \param ir The row index.
* \param parent The parent node as clove::DatasourceValuePointer.
*/        

/*!
* \fn removeColumn()
* \memberof clove::NativeDatasource
* \brief Removes a column.
* \param ic The column index.
* \param parent The parent node as clove::DatasourceValuePointer.
*/        

/*!
* \fn setValue()
* \memberof clove::NativeDatasource
* \brief Sets a new value to a node.
* \param ptr The clove::DatasourceValuePointer.
* \param value The new node value.
*/        

/*!
* \fn setMetadata()
* \memberof clove::NativeDatasource
* \brief Sets a metadata key for a node.
* \param ptr The clove::DatasourceValuePointer.
* \param key The metadata key, like 'icon'.
* \param value The new value.
*/        

/*!
* \fn setRowHeader()
* \memberof clove::NativeDatasource
* \brief Sets a row header configuration.
* \param irow The row index.
* \param parent The parent node as clove::DatasourceValuePointer.
* \param headercfg The header configuration.
*/        

/*!
* \fn setColumnHeader()
* \memberof clove::NativeDatasource
* \brief Sets a column header configuration.
* \param icol The column index.
* \param parent The parent node as clove::DatasourceValuePointer.
* \param headercfg The header configuration.
*/        

/*!
* \class clove::AsyncDatasource
* \extends clove::Datasource
* \extends clove::Headersource
* \brief A clove::Datasource implementation which supports asynchronous data operations like Ajax requests.
*/

/*!
* \fn AsyncDatasource()
* \memberof clove::AsyncDatasource
* The configuration may contain:
* - `'errorhandler'`: Optional `function(error)` as fallback error handler.
* - `'cacheAge'`: The maximum cache age in seconds.
* \param config The async datasource configuration.
*/        
    
/*!
* \fn do_async_pull()
* \memberof clove::AsyncDatasource
* \pure
* \brief Executes an asynchronous data modification.
* 
* Do not call this from outside, use clove::AsyncDatasource::async_pull() instead.
* 
* Override this method in custom implementations.
* \param ptr The clove::DatasourceValuePointer.
* \param requestConfig The request configuration. See clove::AsyncDatasource::async_pull() for details.
*/ 
    
/*!
* \fn do_async_push()
* \memberof clove::AsyncDatasource
* \pure
* \brief Executes an asynchronous data retrieval.
* 
* Do not call this from outside, use clove::AsyncDatasource::async_push() instead.
* 
* Override this method in custom implementations.
* \param ptr The clove::DatasourceValuePointer.
* \param v The new value.
* \param requestConfig The request configuration. See clove::AsyncDatasource::async_pull() for details.
*/ 
    
/*!
* \fn async_pull()
* \memberof clove::AsyncDatasource
* \brief Asynchronously modifies a data value.
* 
* Do not override this in custom implementations, use clove::AsyncDatasource::do_async_pull() instead.
* 
* `requestConfig` may contain:
* - `onsuccess`: A `function()` called when the operation is completed and reflected in this datasource.
* - `onfailed`: A `function(error)` called when an error occurred.
* - `reason`: An optional custom object which can be used by observers for custom decisions.
* \param ptr The clove::DatasourceValuePointer.
* \param requestConfig The request configuration.
*/ 
    
/*!
* \fn async_push()
* \memberof clove::AsyncDatasource
* \brief Asynchronously retrieves a data value.
* 
* Do not override this in custom implementations, use clove::AsyncDatasource::do_async_push() instead.
* \param ptr The clove::DatasourceValuePointer.
* \param v The new value.
* \param requestConfig The request configuration. See clove::AsyncDatasource::async_pull() for details.
*/ 
    
/*!
* \class clove::AjaxAsyncDatasource
* \extends clove::AsyncDatasource
* \extends clove::Headersource
* \brief A clove::AsyncDatasource implementation which uses Ajax requests for data retrieval and modification.
*/    

/*!
* \fn AjaxAsyncDatasource()
* \memberof clove::AjaxAsyncDatasource
* The ajax configuration may contain:
* - `data_pullUrl`: The url for data retrieval.
* - `data_pushUrl`: The url for data modification.
* - `data_pullParams`: Additional parameters for data retrieval.
* - `data_pushParams`: Additional parameters for data modification.
* - `data_params`: Additional parameters.
* - `data_mayHaveChildren`: If the data items might have children.
* - `data_autoPullCharily`: If automatic data fetching should be less aggressive.
* - `params`: Additional parameters.
* - `rootType`: The root type. This controls the prefix of the url and parameter keys to consider 
* - `ajaxAdditionalArgs`: Additional arguments for the ajax request.
* - `answerIdKeyName`: The key name for the identification column used in reading server answers.
* - `requestIdKeyName`: The key name for the identification used in requests.
* - `headerConfigs`: Additional static header configurations.
for fetching the first (and often only) level of objects. 
* Instead of `data_`, other prefixes are allowed as well. The return type name of an retrieved object controls which prefix to use for fetching child objects. 
* \param config The Ajax configuration.
*/        

/*!
* \property OnDataArrived
* \memberof clove::AjaxAsyncDatasource
* \brief Triggered when new data arrived from the server.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \fn getAjaxId()
* \memberof clove::AjaxAsyncDatasource
* \brief Returns the Ajax id for a data cell.
* \param ptr The clove::DatasourceValuePointer.
*/        

/*!
* \class clove::ProxyDatasource
* \extends clove::Datasource
* \extends clove::Headersource
* \brief A clove::Datasource implementation which proxies the content of another datasource in a somehow transformed way.
* 
* Note: This class is an abstract base class. Use one of the subclasses.
*/  

/*!
* \fn setDatasource()
* \memberof clove::ProxyDatasource
* \brief Sets the source datasource.
* \param datasource The source clove::Datasource.
*/        

/*!
* \fn proxyPointerToNativePointer()
* \memberof clove::ProxyDatasource
* \brief Translates a clove::DatasourceValuePointer from this proxy datasource to a one for the source datasource.
* \param proxyptr A value pointer.
*/        

/*!
* \fn nativePointerToProxyPointer()
* \memberof clove::ProxyDatasource
* \brief Translates a clove::DatasourceValuePointer from the source datasource to a one for this proxy datasource.
* \param nativeptr A value pointer.
*/        

/*!
* \fn refresh()
* \memberof clove::ProxyDatasource
* \brief Refreshes the filtering.
* 
* Call this when the outer conditions changed and the filters must be applied again.
*/        

/*!
* \class clove::FilterProxyDatasource
* \extends clove::ProxyDatasource
* \brief A clove::Datasource implementation which filters another clove::Datasource.
*/  

/*!
* \fn FilterProxyDatasource()
* \memberof clove::FilterProxyDatasource
* \param datasource The source clove::Datasource.
* \param rowfilterfct A `function(datasource, idx, parent)` returning `true` if the filter accepts this row (optional).
* \param colfilterfct A `function(datasource, idx, parent)` returning `true` if the filter accepts this column (optional).
*/     

/*!
* \fn setRowFilter()
* \memberof clove::FilterProxyDatasource
* \brief Sets the row filter function.
* \param rowfilterfct The row filter function. See constructor for details.
*/        

/*!
* \fn setColumnFilter()
* \memberof clove::FilterProxyDatasource
* \brief Sets the column filter function.
* \param colfilterfct The column filter function. See constructor for details.
*/        

/*!
* \class clove::SortProxyDatasource
* \extends clove::ProxyDatasource
* \brief A clove::Datasource implementation which sorts another clove::Datasource.
*/  

/*!
* \fn SortProxyDatasource()
* \memberof clove::SortProxyDatasource
* \param datasource The source clove::Datasource.
* \param rowcomparefct A `function(datasource, irow, jrow, parent)` returning a negative value if `irow` points to a row lower than `jrow`, a positive value if it is greater, or 0 if both are equal.
* \param colcomparefct A `function(datasource, icol, jcol, parent)` returning a negative value if `icol` points to a column lower than `jcol`, a positive value if it is greater, or 0 if both are equal.
*/     

/*!
* \fn setRowComparator()
* \memberof clove::SortProxyDatasource
* \brief Sets the row comparator function.
* \param rowcomparefct The row comparator function. See constructor for details.
*/        

/*!
* \fn setColumnComparator()
* \memberof clove::SortProxyDatasource
* \brief Sets the column comparator function.
* \param colcomparefct The column comparator function. See constructor for details.
*/        

/*!
* \class clove::DataBinding
* \brief A data binding. 
* 
* Read the Manual for details.
*/   

/*!
* \fn DataBinding()
* \memberof clove::DataBinding
* \param widget The clove::Widget to bind to the datasource.
* \param propertyName The property name in the widget to bind.
* \param datasource The clove::Datasource to bind to the widget.
* \param ptr The clove::DatasourceValuePointer.
* \param dataDirection See clove::databind().
*/        

/*!
* \fn bind()
* \memberof clove::DataBinding
* \brief Activates the binding.
*/        

/*!
* \fn unbind()
* \memberof clove::DataBinding
* \brief Deactivates the binding.
*/        

/*!
* \fn databind()
* \memberof clove
* \brief Constructs a clove.DataBinding.
* 
* The returned binding is for usage in a widget configuration. It is not completely configured, so it will not
* work out of the box in other situations.
* 
* The databind configuration may contain the following parameters:
* - `datasource`: The clove.Datasource instance to bind.
* - `irow`: The row index.
* - `icol`: The column index.
* - `parent`: The parent node as clove::DatasourceValuePointer.
* - `dataDirection`: One of 'todatasource', 'towidget' or 'bidirectional'.
* \param config The databind configuration.
*/        

/*!
* \class clove::Widget
* \brief Base class for a Clove widget.
* 
* Read the Manual about widgets.
*/        

/*!
* \fn Widget()
* \memberof clove::Widget
* 
* Note: All widgets have a constructor with this signature. The configuration
* parameters are stored in `config`.
* 
* It is typically not required to call those constructors directly. Use clove::build() or 
* some other factory functions (for special situations) instead.
* \param config A widget configuration as in clove::build().
* \param domnode The dom node to use as host for this widget.
*/        

/*!
* \fn declareProperty()
* \memberof clove::Widget
* \brief Declares a widget property.
* 
* This is intended to be called only from inside the widget constructor.
* 
* Read the Manual about widget properties and custom widgets.
* \param k The widget property name.
* \param defaultV The default value.
*/        

/*!
* \fn getProperty()
* \memberof clove::Widget
* \brief General-purpose getter for widget properties.
* 
* Typically used for handling properties, which do not directly belong to the widget class
* but which are helpers for external aspects (e.g. a row index for positioning in a grid).
* 
* The known ones have a dedicated getter and setter, i.e. `x.name()` for `x.getProperty('name')`
* and `x.setName(v)` respectively.
* 
* Read the Manual about widget properties.
* \param k The widget property name.
*/        

/*!
* \fn setProperty()
* \memberof clove::Widget
* \brief General-purpose setter for widget properties.
* 
* See getProperty().
* \param k The widget property name.
* \param v The new value.
*/        

/*!
* \fn bindProperty()
* \memberof clove::Widget
* \brief Binds a DataBinding to a property. Read Manual for details about data bindings.
* \param k The widget property name.
* \param vb The clove.DataBinding to bind. May be 'undefined' for just unbinding.
*/        

/*!
* \fn init()
* \memberof clove::Widget
* \brief Initializes the widget.
* 
* Never override this method in custom widgets. See doresize().
*
* Intended for usage by the infrastructure; never call this method directly.
* \param rootNameScope The root namescope to add this widget to.
*/        

/*!
* \fn doinit()
* \memberof clove::Widget
* \brief Executes late widget initialization.
* 
* Override this method in custom widgets.
*
* Intended for usage by the infrastructure; never call this method directly.
*/        

/*!
* \fn resize()
* \memberof clove::Widget
* \brief Applies the new widget size to internal content.
* 
* Never override this method in custom widgets. See doresize().
* 
* This function is typically called from the parent widget when the size has been changed.
*/        

/*!
* \fn doresize()
* \memberof clove::Widget
* \brief Corrects alignments of internal elements according to the new widget size.
* 
* Override this method in custom widgets.
* 
* Never call this method directly. See resize().
*/        

/*!
* \fn relayout()
* \memberof clove::Widget
* \brief Notifies the parent widget that a new geometry is required.
* 
* This will eventually lead to a resizing according to clove::Widget::getPreferredWidth (et al).
* 
* This function is typically called from within the widget implementation when the content changed and
* from child widgets.
*/        

/*!
* \fn focus()
* \memberof clove::Widget
* \brief Sets the focus to this widget.
*/        

/*!
* \fn isAlive()
* \memberof clove::Widget
* \brief Checks if the widget is still alive (i.e. mounted somewhere in the dom tree).
*/        

/*!
* \fn computeMinimalWidth()
* \memberof clove::Widget
* \brief Computes the minimal width in pixel this widget needs to have.
* 
* Override this method for geometry measurement in custom widgets.
*/        

/*!
* \fn computePreferredWidth()
* \memberof clove::Widget
* \brief Computes the preferred width in pixel this widget needs to have.
* 
* Override this method for geometry measurement in custom widgets.
*/        

/*!
* \fn computeMinimalHeightForWidth()
* \memberof clove::Widget
* \brief Computes the minimal height in pixel this widget needs to have for a given width.
* 
* Override this method for geometry measurement in custom widgets.
* \param w The width in pixel.
*/        

/*!
* \fn computePreferredHeightForWidth()
* \memberof clove::Widget
* \brief Computes the preferred height in pixel this widget needs to have for a given width.
* 
* Override this method for geometry measurement in custom widgets.
* \param w The width in pixel.
*/        

/*!
* \fn getMinimalWidth()
* \memberof clove::Widget
* \brief Returns the minimal width in pixel this widget needs to have.
* 
* Do not override this method in custom widgets.
* This method caches geometry and does some other adjustments.
*/        

/*!
* \fn getPreferredWidth()
* \memberof clove::Widget
* \brief Returns the preferred width in pixel this widget needs to have.
* 
* Do not override this method in custom widgets.
* This method caches geometry and does some other adjustments.
*/        

/*!
* \fn getMinimalHeightForWidth()
* \memberof clove::Widget
* \brief Returns the minimal height in pixel this widget needs to have for a given width.
* 
* Do not override this method in custom widgets.
* This method caches geometry and does some other adjustments.
* \param w The width in pixel.
*/        

/*!
* \fn getPreferredHeightForWidth()
* \memberof clove::Widget
* \brief Returns the preferred height in pixel this widget needs to have for a given width.
* 
* Do not override this method in custom widgets.
* This method caches geometry and does some other adjustments.
* \param w The width in pixel.
*/        

/*!
* \fn addStyleClass()
* \memberof clove::Widget
* \brief Adds the css class `clss` to this widget.
* \param clss A css class name.
*/        

/*!
* \fn removeStyleClass()
* \memberof clove::Widget
* \brief Removes the css class `clss` from this widget.
* \param clss A css class name.
*/        

/*!
* \fn isStyleClass()
* \memberof clove::Widget
* \brief Checks if this widget contains the css class `clss`.
* \param clss A css class name.
*/        

/*!
* \fn setStyleClassAssigned()
* \memberof clove::Widget
* \brief Sets or unsets the css class `clss` to this widget.
* \param clss A css class name.
* \param assigned If to assign or unassign it.
*/   

/*!
* \fn containingNameScope()
* \memberof clove::Widget
* \brief The namescope this widget contains to.
*/        

/*!
* \fn nameScope()
* \memberof clove::Widget
* \brief The own namescope (or the namescope it contains to, if this widget does not bring a new one).
*/        

/*!
* \fn remove()
* \memberof clove::Widget
* \brief Removes this widget.
* 
* Note: This method does not guarantee to remove the widget synchronously, since there might be animations before.
* 
* Use clove::Widget::OnDestroyed for continuing after removal.
* \param removeconfig The removal configuration (optional and only for exotic cases).
*/        

/*!
* \fn effectivelyEnabled()
* \memberof clove::Widget
* \brief If this widget is really enabled (i.e. can interact with the user).
* 
* This checks if the parent widgets are enabled as well.
* See also enabled().
*/    

/*!
* \fn effectiveVisibility()
* \memberof clove::Widget
* \brief If this widget and all parent widgets are visible.
* See also visibility().
*/    

/*!
* \fn childrenWidgets()
* \memberof clove::Widget
* \brief List of the children clove::Widget instances.
*/        

/*!
* \fn parentWidget()
* \memberof clove::Widget
* \brief The parent clove::Widget.
*/        

/*!
* \fn name()
* \memberof clove::Widget
* \brief The name of the widget.
* 
* This name can be used for getting the widget instance later on.
* Read the Manual about namescopes for details.
*/        

/*!
* \fn setName()
* \memberof clove::Widget
* \brief Setter for name().
* \param value The new value.
*/        

/*!
* \fn enabled()
* \memberof clove::Widget
* \brief If this widget is marked as enabled (i.e. can interact with the user).
* 
* This does not check the parents.
* See also effectivelyEnabled().
*/        

/*!
* \fn setEnabled()
* \memberof clove::Widget
* \brief Setter for enabled().
* \param value The new value.
*/        

/*!
* \fn styleClass()
* \memberof clove::Widget
* \brief Custom css class(es).
*/        

/*!
* \fn setStyleClass()
* \memberof clove::Widget
* \brief Setter for styleClass().
* \param value The new value.
*/        

/*!
* \fn style()
* \memberof clove::Widget
* \brief Custom css style string.
*/

/*!
* \fn setStyle()
* \memberof clove::Widget
* \brief Setter for style().
* \param value The new value.
*/        

/*!
* \fn visibility()
* \memberof clove::Widget
* \brief If this widget is visible.
* 
* One of clove::Visible, clove::Invisible and clove::InvisibleCollapsed.
* 
* See also effectiveVisibility().
*/        

/*!
* \fn setVisibility()
* \memberof clove::Widget
* \brief Setter for visibility().
* \param value The new value.
*/        

/*!
* \fn doStandaloneResizing()
* \memberof clove::Widget
* \brief If the widget handles to resize itself as needed.
* 
* This is only needed in exotic situations and by the infrastructure.
*/        

/*!
* \fn setDoStandaloneResizing()
* \memberof clove::Widget
* \brief Setter for doStandaloneResizing().
* \param value The new value.
*/  

/*!
* \fn mayFocus()
* \memberof clove::Widget
* \brief If the widget can have the keyboard focus.
*/        

/*!
* \fn setMayFocus()
* \memberof clove::Widget
* \brief Setter for mayFocus().
* \param value The new value.
*/        

/*!
* \fn hasFocus()
* \memberof clove::Widget
* \brief If the widget has the keyboard focus.
* 
* This also returns true if a child has focus.
*/    

/*!
* \fn innerSize()
* \memberof clove::Widget
* \brief Returns inner width and height of this widget.
*/    

/*!
* \fn outerSize()
* \memberof clove::Widget
* \brief Returns outer width and height of this widget.
*/    

/*!
* \property OnDestroyed
* \memberof clove::Widget
* \brief Triggered when this widget was removed somehow.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnResized
* \memberof clove::Widget
* \brief Triggered when this widget was resized.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnVisibilityChanged
* \memberof clove::Widget
* \brief Triggered when the visibility of this widget changed.
* 
* Only direct changes trigger the event, not when the visibility of a predecessor changed.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnKeyDown
* \memberof clove::Widget
* \brief Triggered when a keyboard key went down.
* 
* This is a clove.Event instance. See Manual for details.
*/   

/*!
* \property OnKeyUp
* \memberof clove::Widget
* \brief Triggered when a keyboard key came up.
* 
* This is a clove.Event instance. See Manual for details.
*/   

/*!
* \property OnKeyPress
* \memberof clove::Widget
* \brief Triggered when a keyboard key was pressed.
* 
* Note that this event does not work for all keys in all browsers!
* 
* This is a clove.Event instance. See Manual for details.
*/   

/*!
* \class clove::Label
* \extends clove::Widget
* \brief A text label.
*/        

/*!
* \fn label()
* \memberof clove::Label
* \brief The label text.
* 
* For rich content, use htmlContent() instead.
*/        

/*!
* \fn setLabel()
* \memberof clove::Label
* \brief Setter for label().
* \param value The new value.
*/        

/*!
* \fn htmlContent()
* \memberof clove::Label
* \brief The label content as html.
* 
* For plain text, use label() instead.
*/        

/*!
* \fn setHtmlContent()
* \memberof clove::Label
* \brief Setter for htmlContent().
* \param value The new value.
*/        

/*!
* \fn focusBuddy()
* \memberof clove::Label
* \brief A clove::Widget or a widget name for a focus buddy.
* 
* Whenever the user clicks on this label, it will focus the buddy.
*/        

/*!
* \fn setFocusBuddy()
* \memberof clove::Label
* \brief Setter for focusBuddy().
* \param value The new value.
*/        

/*!
* \class clove::Spacer
* \extends clove::Widget
* \brief A spacer widget for filling gaps in a layout.
* 
* It is essentially just a widget which does nothing, but respects the
* layouting properties like all widgets. Add them to a layout and configure
* them according to your sizing requirements.
*/        

/*!
* \class clove::HtmlView
* \extends clove::Widget
* \brief A host for arbitrary html content.
*/        

/*!
* \fn contentRoot()
* \memberof clove::HtmlView
* \brief The root dom node of the html content.
*/        

/*!
* \fn setContentRoot()
* \memberof clove::HtmlView
* \brief Setter for contentRoot().
* \param value The new value.
*/        

/*!
* \class clove::Layout
* \brief A mixin for a widget classes which align child widgets in some way.
* 
* This is a base type and has some more interesting sub-classes.
*/        

/*!
* \fn children()
* \memberof clove::Layout
* \brief List of all child widgets in this layout as widget configurations like in clove::build().
*/        

/*!
* \fn setChildren()
* \memberof clove::Layout
* \brief Setter for children().
* \param value The new value.
*/        

/*!
* \fn addChild()
* \memberof clove::Layout
* \brief Adds a new child widget to the layout.
* \param value The new widget as widget configuration like for clove::build().
*/        

/*!
* \fn clearChilds()
* \memberof clove::Layout
* \brief Removes all childs from the layout.
*/        

/*!
* \class clove::GridLayout
* \extends clove::Layout
* \brief A mixin for grid layout implementations.
*/        

/*!
* \fn insertRow()
* \memberof clove::GridLayout
* \brief Inserts a new empty row to the grid.
* \param i The row index.
*/        

/*!
* \fn insertColumn()
* \memberof clove::GridLayout
* \brief Inserts a new empty column to the grid.
* \param i The column index.
*/        

/*!
* \fn removeRow()
* \memberof clove::GridLayout
* \brief Removes a row from the grid.
* \param i The row index.
*/        

/*!
* \fn removeColumn()
* \memberof clove::GridLayout
* \brief Removes a column from the grid.
* \param i The column index.
*/        

/*!
* \class clove::Grid
* \extends clove::Widget
* \extends clove::GridLayout
* \brief A container for aligning child widgets in a grid.
* 
* Set the `row` and `col` property in the child widget configurations to
* some numbers for assigning them to cells.
*/        

/*!
* \class clove::StackLayout
* \extends clove::GridLayout
* \brief A mixin for stack layout implementations.
*/        

/*!
* \class clove::HorizontalStack
* \extends clove::Widget
* \extends clove::StackLayout
* \brief A container which stacks child widgets column-wise.
*/        

/*!
* \fn cols()
* \memberof clove::HorizontalStack
* \brief The list of child widgets as widget configurations like for clove::build().
*/        

/*!
* \fn setCols()
* \memberof clove::HorizontalStack
* \brief Setter for cols().
* \param value The new value.
*/        

/*!
* \class clove::VerticalStack
* \extends clove::Widget
* \extends clove::StackLayout
* \brief A container which stacks child widgets row-wise.
*/        

/*!
* \fn rows()
* \memberof clove::VerticalStack
* \brief The list of child widgets as widget configurations like for clove::build().
*/        

/*!
* \fn setRows()
* \memberof clove::VerticalStack
* \brief Setter for rows().
* \param value The new value.
*/        

/*!
* \class clove::Button
* \extends clove::Widget
* \brief A button allows the user to trigger a particular program event.
*/        

/*!
* \fn label()
* \memberof clove::Button
* \brief The label text.
*/        

/*!
* \fn setLabel()
* \memberof clove::Button
* \brief Setter for label().
* \param value The new value.
*/        

/*!
* \fn icon()
* \memberof clove::Button
* \brief The optional clove::Icon button icon.
*/        

/*!
* \fn setIcon()
* \memberof clove::Button
* \brief Setter for icon().
* \param value The new value.
*/        

/*!
* \fn checkable()
* \memberof clove::Button
* \brief If the button is checkable (i.e. has a checked-flag).
* 
* If it has, the checked-flag is controlled by checked().
*/        

/*!
* \fn setCheckable()
* \memberof clove::Button
* \brief Setter for checkable().
* \param value The new value.
*/        

/*!
* \fn checked()
* \memberof clove::Button
* \brief For a checkable button, return if it is checked.
* 
* See also checkable().
* 
* User interactions do not toggle the checked flag by default. This must be scripted in own event handlers as needed.
*/        

/*!
* \fn setChecked()
* \memberof clove::Button
* \brief Setter for checked().
* \param value The new value.
*/        

/*!
* \property OnClicked
* \memberof clove::Button
* \brief Triggered when the user clicks on this button.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \class clove::PopupMenuButton
* \extends clove::Button
* \brief A special clove::Button which opens a popup menu when it is triggered.
*/        

/*!
* \fn actions()
* \memberof clove::PopupMenuButton
* \brief The menu actions. Same as for clove::Menubar::actions().
*/        

/*!
* \fn setActions()
* \memberof clove::PopupMenuButton
* \brief Setter for actions().
* \param value The new value.
*/        

/*!
* \property OnActionTriggered
* \memberof clove::PopupMenuButton
* \brief Triggered when a menu action was chosen by the user for execution.
* 
* The event arguments contain the selected action name in `action`.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property MenuSeparator
* \memberof clove
* \brief A menu separator for usage in clove::AbstractMenu::actions().
*/        

/*!
* \class clove::AbstractMenu
* \extends clove::Widget
* \brief A base class for widgets which present a menu of actions to the user.
*/        

/*!
* \fn actions()
* \memberof clove::AbstractMenu
* \brief The actions to be shown in this menu.
* 
* It is a list of action configuration objects, each having a structure like
* `{name:'myaction', label:'My menu action'}`. It can have a list of sub-items in the
* `subactions` property.
* 
* Instead of a simple list, it may also be a clove::Datasource with the action configuration
* structures aligned in rows.
* 
* Use clove::MenuSeparator for a separator.
* 
* One action allows this properties:
* - `name`: The action name.
* - `label`: The label string.
* - `icon`: A clove::Icon.
* - `disabled`: If it is disabled.
* - `invisible`: If it is invisible.
* - `checkable`: If it is checkable.
* - `checked`: If it is checked.
*/        

/*!
* \fn setActions()
* \memberof clove::AbstractMenu
* \brief Setter for actions().
* \param value The new value.
*/       

/*!
* \property OnActionTriggered
* \memberof clove::AbstractMenu
* \brief Triggered when a menu action was chosen by the user for execution.
* 
* The event arguments contain the selected action name in `action`.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \class clove::Menubar
* \extends clove::AbstractMenu
* \brief A menu bar.
*/        

/*!
* \class clove::PopupMenu
* \extends clove::AbstractMenu
* \brief A popup menu (i.e. a box with vertically listed actions).
*/        

/*!
* \fn expanderIndicatorDirection()
* \memberof clove::PopupMenu
* \brief If the expander indicator is to be shown on the `'right'` or `'left'` side.
*/        

/*!
* \fn setExpanderIndicatorDirection()
* \memberof clove::PopupMenu
* \brief Setter for expanderIndicatorDirection().
* \param value The new value.
*/        

/*!
* \fn static show()
* \memberof clove::PopupMenu
* \brief Shows a popup menu.
* 
* The root view of `menu` is expected to be a clove::PopupMenu (or a subclass).
* \param menu The widget configuration, as for clove::build(), which specifies the popup menu.
* \param showconfig A configuration object which specifies some presentation aspects.
*/ 

/*!
* \class clove::Toolbar
* \extends clove::Widget
* \brief A toolbar has header texts and a menu bar in a perceptible visual bar, typically used on top of a user interface with all available width.
*/ 

/*!
* \fn actions()
* \memberof clove::Toolbar
* \brief Menu actions.
* 
* See clove::Menubar::actions().
*/        

/*!
* \fn setActions()
* \memberof clove::Toolbar
* \brief Setter for actions().
* \param value The new value.
*/    

/*!
* \fn head1()
* \memberof clove::Toolbar
* \brief The 1st header text.
*/        

/*!
* \fn setHead1()
* \memberof clove::Toolbar
* \brief Setter for head1().
* \param value The new value.
*/    

/*!
* \fn head2()
* \memberof clove::Toolbar
* \brief The 2nd header text.
*/        

/*!
* \fn setHead2()
* \memberof clove::Toolbar
* \brief Setter for head2().
* \param value The new value.
*/    

/*!
* \fn icon()
* \memberof clove::Toolbar
* \brief The header icon as clove::Icon.
*/        

/*!
* \fn setIcon()
* \memberof clove::Toolbar
* \brief Setter for icon().
* \param value The new value.
*/        

/*!
* \property OnActionTriggered
* \memberof clove::Toolbar
* \brief Triggered when a menu action was chosen by the user for execution.
* 
* The event arguments contain the selected action name in `action`.
* 
* This is a clove.Event instance. See Manual for details.
*/              

/*!
* \class clove::ScrollView
* \extends clove::Widget
* \brief A container for making the children scrollable.
*/  

/*!
* \fn body()
* \memberof clove::ScrollView
* \brief The inner widget as widget configuration like for clove::build().
*/        

/*!
* \fn setBody()
* \memberof clove::ScrollView
* \brief Setter for body().
* \param value The new value.
*/        

/*!
* \fn bodyWidget()
* \memberof clove::ScrollView
* \brief Returns the body as clove::Widget.
*/        

/*!
* \fn bodySize()
* \memberof clove::ScrollView
* \brief The (outer) size of the body widget.
*/        

/*!
* \fn verticalScrollPosition()
* \memberof clove::ScrollView
* \brief The vertical scroll position.
*/        

/*!
* \fn setVerticalScrollPosition()
* \memberof clove::ScrollView
* \brief Setter for verticalScrollPosition().
* \param value The new value.
*/        

/*!
* \fn horizontalScrollPosition()
* \memberof clove::ScrollView
* \brief The horizontal scroll position.
*/        

/*!
* \fn setHorizontalScrollPosition()
* \memberof clove::ScrollView
* \brief Setter for horizontalScrollPosition().
* \param value The new value.
*/        

/*!
* \class clove::MainView
* \extends clove::Widget
* \brief A typical main view, including a clove::Toolbar and a container for an inner body widget.
*/        

/*!
* \fn sidebar()
* \memberof clove::MainView
* \brief The sidebar widget as widget configuration like for clove::build().
*/        

/*!
* \fn setSidebar()
* \memberof clove::MainView
* \brief Setter for sidebar().
* \param value The new value.
*/        

/*!
* \fn bodyLeft()
* \memberof clove::MainView
* \brief The left inner widget as widget configuration like for clove::build().
*/        

/*!
* \fn setBodyLeft()
* \memberof clove::MainView
* \brief Setter for bodyLeft().
* \param value The new value.
*/        

/*!
* \fn bodyRight()
* \memberof clove::MainView
* \brief The right inner widget as widget configuration like for clove::build().
*/        

/*!
* \fn setBodyRight()
* \memberof clove::MainView
* \brief Setter for bodyRight().
* \param value The new value.
*/        

/*!
* \fn bodyLeftViewActionLabel()
* \memberof clove::MainView
* \brief The title string for the left inner widget.
*/        

/*!
* \fn setBodyLeftViewActionLabel()
* \memberof clove::MainView
* \brief Setter for bodyLeftViewActionLabel().
* \param value The new value.
*/        

/*!
* \fn bodyRightViewActionLabel()
* \memberof clove::MainView
* \brief The title string for the right inner widget.
*/        

/*!
* \fn setBodyRightViewActionLabel()
* \memberof clove::MainView
* \brief Setter for bodyRightViewActionLabel().
* \param value The new value.
*/        

/*!
* \fn splitterPosition()
* \memberof clove::MainView
* \brief The position of the splitter between the left and right inner widget as number between `0.0` and `1.0`.
*/        

/*!
* \fn setSplitterPosition()
* \memberof clove::MainView
* \brief Setter for splitterPosition().
* \param value The new value.
*/        

/*!
* \fn switchToRightBody()
* \memberof clove::MainView
* \brief If in small mode, it switches the view to the right body.
*/        

/*!
* \fn switchToLeftBody()
* \memberof clove::MainView
* \brief If in small mode, it switches the view to the left body.
*/        

/*!
* \fn head1()
* \memberof clove::MainView
* \brief The 1st header text.
*/        

/*!
* \fn setHead1()
* \memberof clove::MainView
* \brief Setter for head1().
* \param value The new value.
*/        

/*!
* \fn head2()
* \memberof clove::MainView
* \brief The 2nd header text.
*/        

/*!
* \fn setHead2()
* \memberof clove::MainView
* \brief Setter for head2().
* \param value The new value.
*/        

/*!
* \fn icon()
* \memberof clove::MainView
* \brief The header icon as clove::Icon.
*/        

/*!
* \fn setIcon()
* \memberof clove::MainView
* \brief Setter for icon().
* \param value The new value.
*/        

/*!
* \fn actions()
* \memberof clove::MainView
* \brief Menu actions.
* 
* See clove::Menubar::actions().
*/        

/*!
* \fn setActions()
* \memberof clove::MainView
* \brief Setter for actions().
* \param value The new value.
*/        

/*!
* \fn showOnly()
* \memberof clove::MainView
* \brief Specifies if to forcefully show just one of the two main panels (`0`, `1`, `undefined`).
*/        

/*!
* \fn setShowOnly()
* \memberof clove::MainView
* \brief Setter for showOnly().
* \param value The new value.
*/        

/*!
* \class clove::ModalPanel
* \extends clove::Widget
* \brief A surface for implementing modality.
* 
* For a certain time, it will be inserted before the main user interface (i.e. higher on z-axis) in order to block user interaction with it.
* 
* It is typically not required to use it directly. Try clove::Dialog::show() and clove::utils::popup() before.
*/       

/*!
* \fn fullyTransparent()
* \memberof clove::ModalPanel
* \brief If the modal panel is fully transparent (i.e. not really visible) instead of semi-transparent.
*/        

/*!
* \fn setFullyTransparent()
* \memberof clove::ModalPanel
* \brief Setter for fullyTransparent().
* \param value The new value.
*/        

/*!
* \property OnClicked
* \memberof clove::ModalPanel
* \brief Triggered when the user clicks on this modal panel.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \class clove::Dialog
* \extends clove::Widget
* \brief A dialog is a container for a new sub user interface, decorated with a border and a title bar.
*/        

/*!
* \fn body()
* \memberof clove::Dialog
* \brief The inner widget as widget configuration like for clove::build().
*/        

/*!
* \fn setBody()
* \memberof clove::Dialog
* \brief Setter for body().
* \param value The new value.
*/        

/*!
* \fn title()
* \memberof clove::Dialog
* \brief The dialog title text.
*/        

/*!
* \fn setTitle()
* \memberof clove::Dialog
* \brief Setter for title().
* \param value The new value.
*/        

/*!
* \fn titleicon()
* \memberof clove::Dialog
* \brief The dialog title icon as clove::Icon.
*/        

/*!
* \fn setTitleicon()
* \memberof clove::Dialog
* \brief Setter for titleicon().
* \param value The new value.
*/        

/*!
* \fn close()
* \memberof clove::Dialog
* \brief Closes and removes this dialog.
*/        

/*!
* \fn static show()
* \memberof clove::Dialog
* \brief Shows a dialog.
* 
* The root view of `dlg` is expected to be a clove::Dialog (or a subclass).
* \param dlg The widget configuration, as for clove::build(), which specifies the dialog.
* \param showconfig A configuration object which specifies some presentation aspects.
*/        
     
/*!
* \class clove::DataView
* \extends clove::Widget
* \brief Base class for some views which shows hierarchical data from a clove::Datasource.
* 
* Read the Manual for details. 
* 
* Although this is a fully-functional and implemented class, you should consider using one of the subclasses.
*/        

/*!
* \fn datasource()
* \memberof clove::DataView
* \brief The clove::Datasource for this view. 
* 
* This provides the actual data to display.
* See clove::NativeDatasource for a ready-to-use implementation.
*/        

/*!
* \fn setDatasource()
* \memberof clove::DataView
* \brief Setter for datasource().
* \param value The new value.
*/        

/*!
* \fn dataViewProcessor()
* \memberof clove::DataView
* \brief An optional `function(ptr, value)` for processing a datasource value to a display string.
*/        

/*!
* \fn setDataViewProcessor()
* \memberof clove::DataView
* \brief Setter for dataViewProcessor().
* \param value The new value.
*/    

/*!
* \fn cellGenerator()
* \memberof clove::DataView
* \brief A generator function for complex cell content.
* 
* This method is called for each cell with `(clove::Widget, clove::DatasourceValuePointer)`. It may return a widget configuration as for clove::build().
* If it does, the cell is constructed with this widget as content. The content must define an `update(clove::Widget, clove::DatasourceValuePointer, value)` method,
* which takes the cell datasource value and configures the inner widget according to that.
*/        

/*!
* \fn setCellGenerator()
* \memberof clove::DataView
* \brief Setter for cellGenerator().
* \param value The new value.
*/    

/*!
* \fn alwaysAllocateExpanderSpace()
* \memberof clove::DataView
* \brief If some space is allocated for an expander even for cells without children.
*/        

/*!
* \fn setAlwaysAllocateExpanderSpace()
* \memberof clove::DataView
* \brief Setter for alwaysAllocateExpanderSpace().
* \param value The new value.
*/    

/*!
* \fn showOnlyFirstColumn()
* \memberof clove::DataView
* \brief If to show only the first column and hide the other ones.
*/        

/*!
* \fn setShowOnlyFirstColumn()
* \memberof clove::DataView
* \brief Setter for showOnlyFirstColumn().
* \param value The new value.
*/    

/*!
* \fn hideExpanders()
* \memberof clove::DataView
* \brief If to hide all expanders.
*/        

/*!
* \fn setHideExpanders()
* \memberof clove::DataView
* \brief Setter for hideExpanders().
* \param value The new value.
*/    
     
/*!
* \fn allowSelection()
* \memberof clove::DataView
* \brief If it is allowed to select nodes.
* 
* This is always a single selection. For something like multi-selection, please check
* allowChecking().
*/        

/*!
* \fn setAllowSelection()
* \memberof clove::DataView
* \brief Setter for allowSelection().
* \param value The new value.
*/    

/*!
* \fn allowChecking()
* \memberof clove::DataView
* \brief If it is allowed to check cells.
* 
* This is a way to select 0..n data cells. For a single-selection, please check allowSelection().
*/        

/*!
* \fn setAllowChecking()
* \memberof clove::DataView
* \brief Setter for allowChecking().
* \param value The new value.
*/    

/*!
* \fn granularity()
* \memberof clove::DataView
* \brief The granularity for stuff like selection and checking.
* 
* Either `'cell'` or `'row'`.
*/        

/*!
* \fn setGranularity()
* \memberof clove::DataView
* \brief Setter for granularity().
* \param value The new value.
*/    

/*!
* \fn selectCell()
* \memberof clove::DataView
* \brief Selects a cell.
* \param ptr The clove::DatasourceValuePointer.
*/        

/*!
* \fn isCellSelected()
* \memberof clove::DataView
* \brief Checks if a given cell is selected.
* \param ptr The clove::DatasourceValuePointer.
*/        

/*!
* \fn checkedCells()
* \memberof clove::DataView
* \brief Returns the checked cells as list of clove::DatasourceValuePointer.
*/        

/*!
* \fn setCheckedCells()
* \memberof clove::DataView
* \brief Seturns the checked cells.
* \param ptrs A list of clove::DatasourceValuePointer.
*/        

/*!
* \fn isCellChecked()
* \memberof clove::DataView
* \brief Checks if a given cell is checked.
* \param ptr The clove::DatasourceValuePointer.
*/ 

/*!
* \fn setCellChecked()
* \memberof clove::DataView
* \brief Sets if a given cell is checked.
* \param ptr The clove::DatasourceValuePointer.
* \param val If the cells shall be checked.
*/        

/*!
* \fn selection()
* \pure
* \memberof clove::DataView
* \brief Returns the selected item as clove::DatasourceValuePointer.
*/        

/*!
* \fn headersource()
* \memberof clove::DataView
* \brief The clove::Headersource providing row and column header configurations.
*/        

/*!
* \fn setHeadersource()
* \memberof clove::DataView
* \brief Setter for headersource().
* \param value The new value.
*/        

/*!
* \fn gridVisible()
* \memberof clove::DataView
* \brief If to show a cell grid.
*/        

/*!
* \fn setGridVisible()
* \memberof clove::DataView
* \brief Setter for gridVisible().
* \param value The new value.
*/        

/*!
* \fn nodeActivationNeedsDoubleClick()
* \memberof clove::DataView
* \brief If node activation needs a double-click.
* 
* Note: If you set this, you should find alternative ways for users
* of mobile devices!
*/        

/*!
* \fn setNodeActivationNeedsDoubleClick()
* \memberof clove::DataView
* \brief Setter for nodeActivationNeedsDoubleClick().
* \param value The new value.
*/        

/*!
* \fn expandCell()
* \memberof clove::DataView
* \brief Expands a given cell.
* \param ptr The clove::DatasourceValuePointer.
*/  

/*!
* \fn expandCellRecursive()
* \memberof clove::DataView
* \brief Expands a given cell and all parents.
* \param ptr The clove::DatasourceValuePointer.
*/  

/*!
* \fn collapseCell()
* \memberof clove::DataView
* \brief Collapses a given cell.
* \param ptr The clove::DatasourceValuePointer.
*/        

/*!
* \fn isCellExpanded()
* \memberof clove::DataView
* \brief Checks if a given cell is expanded.
* \param ptr The clove::DatasourceValuePointer.
*/        

/*!
* \fn editCell()
* \memberof clove::DataView
* \brief Triggers the edit mode for a cell.
* 
* Only works if a method `_getdomcell(ir, ic, parent)` returns a dom node.
* \param ptr The clove::DatasourceValuePointer.
*/        

/*!
* \property OnSelectionChanged
* \memberof clove::DataView
* \brief Triggered when the selection in the view changes.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \class clove::TableView
* \extends clove::DataView
* \brief A view which shows a clove::Datasource in a table.
*/      

/*!
* \class clove::ListView
* \extends clove::DataView
* \brief A view which shows a clove::Datasource in a list.
*/        

/*!
* \class clove::TreeView
* \extends clove::DataView
* \brief A view which shows a clove::Datasource in a tree.
*/        

/*!
* \class clove::CheckButton
* \extends clove::Widget
* \brief A check button.
* 
* The user can choose to check and uncheck freely (in contrast to clove::RadioButton).
*/        

/*!
* \fn label()
* \memberof clove::CheckButton
* \brief The label text.
*/        

/*!
* \fn setLabel()
* \memberof clove::CheckButton
* \brief Setter for label().
* \param value The new value.
*/        

/*!
* \fn checked()
* \memberof clove::CheckButton
* \brief If this check button is checked.
*/        

/*!
* \fn setChecked()
* \memberof clove::CheckButton
* \brief Setter for checked().
* \param value The new value.
*/        

/*!
* \property OnChanged
* \memberof clove::CheckButton
* \brief Triggered when this check button was selected.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnClicked
* \memberof clove::CheckButton
* \brief Triggered when this check button was selected. Same as OnChanged.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \class clove::RadioGroup
* \brief Groups some clove::RadioButton together in a logical way, so
* the user can select exactly one of them.
* 
* Assign buttons to a group by setting clove::RadioButton::group().
*/        

/*!
* \fn RadioGroup()
* \memberof clove::RadioGroup
*/        

/*!
* \fn selected()
* \memberof clove::RadioGroup
* \brief Returns the clove::RadioButton which is currently selected in this group.
*/        

/*!
* \fn select()
* \memberof clove::RadioGroup
* \brief Selects a button in the group.
* \param w The clove::RadioButton to select.
*/        

/*!
* \property OnChanged
* \memberof clove::RadioGroup
* \brief Triggered when another button was selected in this group.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \class clove::RadioButton
* \extends clove::Widget
* \brief A radio button.
* 
* Typically some radio buttons are assigned to one clove::RadioGroup so the
* user can choose one of them (while a clove::CheckButton allows 0..n choices).
*/        

/*!
* \fn label()
* \memberof clove::RadioButton
* \brief The label text.
*/        

/*!
* \fn setLabel()
* \memberof clove::RadioButton
* \brief Setter for label().
* \param value The new value.
*/        

/*!
* \fn checked()
* \memberof clove::RadioButton
* \brief If this radio button is checked.
* 
* See also clove::RadioGroup::selected().
*/        

/*!
* \fn setChecked()
* \memberof clove::RadioButton
* \brief Setter for checked().
* \param value The new value.
*/        

/*!
* \fn group()
* \memberof clove::RadioButton
* \brief The clove::RadioGroup this radio button belongs to.
*/        

/*!
* \fn setGroup()
* \memberof clove::RadioButton
* \brief Setter for group().
* \param value The new value.
*/        

/*!
* \property OnChanged
* \memberof clove::RadioButton
* \brief Triggered when this radio button was selected.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnClicked
* \memberof clove::RadioButton
* \brief Triggered when this radio button was selected. Same as OnChanged.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \class clove::EditBox
* \extends clove::Widget
* \brief A single-line box for text input from the user.
*/    

/*!
* \fn readOnly()
* \memberof clove::EditBox
* \brief If the edit box is read-only or editable by the user.
*/        

/*!
* \fn setReadOnly()
* \memberof clove::EditBox
* \brief Setter for readOnly().
* \param value The new value.
*/        

/*!
* \fn text()
* \memberof clove::EditBox
* \brief The current text.
*/        

/*!
* \fn setText()
* \memberof clove::EditBox
* \brief Setter for text().
* \param value The new value.
*/        

/*!
* \fn hintText()
* \memberof clove::EditBox
* \brief The hint text.
* 
* Typically contains something like a label text. 
* It is shown as a hint when the box is empty.
*/        

/*!
* \fn setHintText()
* \memberof clove::EditBox
* \brief Setter for hintText().
* \param value The new value.
*/        

/*!
* \fn autocompletionItems()
* \memberof clove::EditBox
* \brief A clove.Datasource with a list of autocompletion items to popup.
* 
* It is allowed to observe the `text` in order to generate live content for this list.
*/        

/*!
* \fn setAutocompletionItems()
* \memberof clove::EditBox
* \brief Setter for autocompletionItems().
* \param value The new value.
*/  

/*!
* \fn autocompletionFilter()
* \memberof clove::EditBox
* \brief How to filter the autocompletion list.
* 
* Either `'startswith'`, `'contains'`, `function(itemtext, boxtext)` or `undefined`.
*/        

/*!
* \fn setAutocompletionFilter()
* \memberof clove::EditBox
* \brief Setter for autocompletionFilter().
* \param value The new value.
*/  

/*!
* \fn autocompletionOpenForNoText()
* \memberof clove::EditBox
* \brief If to show the autocompletion list when the edit box is empty.
*/        

/*!
* \fn setAutocompletionOpenForNoText()
* \memberof clove::EditBox
* \brief Setter for autocompletionOpenForNoText().
* \param value The new value.
*/  

/*!
* \fn setTextSelection()
* \memberof clove::EditBox
* \brief Selects the specified part of the text.
* \param ifrom The selection begin index.
* \param ito The selection end index.
*/  

/*!
* \property OnChanged
* \memberof clove::EditBox
* \brief Triggered when the text was changed.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnPopupTextSelected
* \memberof clove::EditBox
* \brief Triggered when a text was selected from the popup.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \class clove::EditComboBox
* \extends clove::EditBox
* \brief A single-line box for text input from the user with an additional popup list of value recommendations.
*/        

/*!
* \fn popupItems()
* \memberof clove::EditComboBox
* \brief A clove.Datasource with a list of items for the popup.
*/        

/*!
* \fn setPopupItems()
* \memberof clove::EditComboBox
* \brief Setter for popupItems().
* \param value The new value.
*/        

/*!
* \class clove::DropdownBox
* \extends clove::EditComboBox
* \brief A single-line text box (without edit functionality) with a popup list of values to select from.
*/        

/*!
* \class clove::NumericEditBox
* \extends clove::EditBox
* \brief A text box with up/down buttons for numbers.
*/        

/*!
* \fn min()
* \memberof clove::NumericEditBox
* \brief The minimum value.
*/        

/*!
* \fn setMin()
* \memberof clove::NumericEditBox
* \brief Setter for min().
* \param value The new value.
*/        

/*!
* \fn max()
* \memberof clove::NumericEditBox
* \brief The maximum value.
*/        

/*!
* \fn setMax()
* \memberof clove::NumericEditBox
* \brief Setter for max().
* \param value The new value.
*/        

/*!
* \fn stepsize()
* \memberof clove::NumericEditBox
* \brief The step size.
*/        

/*!
* \fn setStepsize()
* \memberof clove::NumericEditBox
* \brief Setter for stepsize().
* \param value The new value.
*/        

/*!
* \fn value()
* \memberof clove::NumericEditBox
* \brief The current numeric value.
*/        

/*!
* \fn setValue()
* \memberof clove::NumericEditBox
* \brief Setter for value().
* \param value The new value.
*/        

/*!
* \class clove::MultilineEditBox
* \extends clove::EditBox
* \brief A multi-line box for text input from the user.
*/        

/*!
* \class clove::PasswordEditBox
* \extends clove::EditBox
* \brief A box for password input from the user.
*/        

/*!
* \class clove::ImageView
* \extends clove::Widget
* \brief Shows an image.
* 
* This does not contain any viewer controls like zoom buttons.
*/     

/*!
* \fn source()
* \memberof clove::ImageView
* \brief The image source URL.
*/        

/*!
* \fn setSource()
* \memberof clove::ImageView
* \brief Setter for source().
* \param value The new value.
*/        

/*!
* \fn zoom()
* \memberof clove::ImageView
* \brief The zoom level.
* 
* 1.0 is normal, larger values zoom in, smaller values zoom out.
* Specify 'auto' for automatically adjusting it to the available room.
*/        

/*!
* \fn setZoom()
* \memberof clove::ImageView
* \brief Setter for zoom().
* \param value The new value.
*/        

/*!
* \fn keepAspectRatio()
* \memberof clove::ImageView
* \brief For 'auto' zoom: Whether to keep aspect ratio of the image.
*/        

/*!
* \fn setKeepAspectRatio()
* \memberof clove::ImageView
* \brief Setter for keepAspectRatio().
* \param value The new value.
*/        

/*!
* \class clove::Icon
* \brief An icon.
* 
* Can be shown at many places, like in buttons, menus or clove::IconView.
* 
* Construct instances with clove::Icon::byUrl() or clove::Icon::bySymbol().
*/     

/*!
* \fn static byUrl()
* \memberof clove::Icon
* \brief Creates a clove::Icon by a url pointing to an image.
* \param url The image url.
*/        

/*!
* \fn static bySymbol()
* \memberof clove::Icon
* \brief Creates a clove::Icon by a text character.
* \param chr The text character.
*/        

/*!
* \fn createHtml()
* \memberof clove::Icon
* \brief Creates an html representation.
* 
* This method is used by widget implementations.
* \param height The icon height in pixels.
*/        

/*!
* \class clove::IconView
* \extends clove::Widget
* \brief Shows an icon.
* 
* The icon can come from an image file or from a Unicode symbol.
*/     

/*!
* \fn size()
* \memberof clove::IconView
* \brief The icon size as css length.
*/        

/*!
* \fn setSize()
* \memberof clove::IconView
* \brief Setter for size().
* \param value The new value.
*/        

/*!
* \fn icon()
* \memberof clove::IconView
* \brief The clove::Icon to show.
*/        

/*!
* \fn setIcon()
* \memberof clove::IconView
* \brief Setter for icon().
* \param value The new value.
*/        

/*!
* \class clove::TabView
* \extends clove::Widget
* \brief A tabbed container.
* 
* Typically used for grouping and folding some user interface parts together for
* better space efficiency and higher usability.
*/

/*!
* \fn tabs()
* \memberof clove::TabView
* \brief The list of tabs. 
* 
* Each tab is a tuple of (I) a widget configuration, like for clove::build() and (II) a tab configuration, which holds infos like the tab header text.
* So, for example, one item in that list could be `[{view:'Label', label:...}, {head:'Tab1'}]`. See also addTab().
*/        

/*!
* \fn setTabs()
* \memberof clove::TabView
* \brief Setter for tabs().
* \param value The new value.
*/        

/*!
* \fn currentTab()
* \memberof clove::TabView
* \brief The index of the currently selected tab.
*/        

/*!
* \fn setCurrentTab()
* \memberof clove::TabView
* \brief Setter for currentTab().
* \param value The new value.
*/        

/*!
* \fn userMayAddTabs()
* \memberof clove::TabView
* \brief If to show a button for adding new tabs.
* 
* If `true`, implement a handler for OnTabCreationRequested and create a tab with addTab() there.
*/        

/*!
* \fn setUserMayAddTabs()
* \memberof clove::TabView
* \brief Setter for userMayAddTabs().
* \param value The new value.
*/        

/*!
* \property OnTabCreationRequested
* \memberof clove::TabView
* \brief Triggered when the user requested a new tab.
* 
* See also userMayAddTabs().
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \fn addTab()
* \memberof clove::TabView
* \brief Adds a new tab.
* 
* The tab configuration may contain:
* - `head`: The tab header text.
* - `icon`: The tab icon as clove::Icon.
* - `userMayCloseTab`: If the user may close this tab.
* 
* This method returns a clove::Widget which can be used for getting subwidgets or removing the tab.
* \param config A widget configuration like for clove::build().
* \param tabconfig A tab configuration.
*/        

/*!
* \class clove::DateBox
* \extends clove::EditBox
* \brief A user input box for date input.
* 
* The actual behavior depends on the browser.
*/        

/*!
* \fn date()
* \memberof clove::DateBox
* \brief The date value in the box as JavaScript Date.
*/        

/*!
* \fn setDate()
* \memberof clove::DateBox
* \brief Setter for date().
* \param value The new value.
*/        

/*!
* \class clove::TimeBox
* \extends clove::EditBox
* \brief A user input box for time input.
* 
* The actual behavior depends on the browser.
*/        

/*!
* \fn time()
* \memberof clove::TimeBox
* \brief The time value in the box as JavaScript Date.
*/        

/*!
* \fn setTime()
* \memberof clove::TimeBox
* \brief Setter for time().
* \param value The new value.
*/        

/*!
* \class clove::RichEditBox
* \extends clove::Widget
* \brief A user input box for text with rich formatting.
* 
* This widget allows to be controlled from external toolbars.
* For a widget including a formatting toolbar, see clove::RichEdit.
*/        

/*!
* \fn htmlContent()
* \memberof clove::RichEditBox
* \brief The content text as html string.
*/        

/*!
* \fn setHtmlContent()
* \memberof clove::RichEditBox
* \brief Setter for htmlContent().
* \param value The new value.
*/        

/*!
* \fn contentHtmlNode()
* \memberof clove::RichEditBox
* \brief Returns the html content dom node.
*/        

/*!
* \fn toggleSelectionBold()
* \memberof clove::RichEditBox
* \brief Toggles the bold-flag for the current selection.
*/        

/*!
* \fn toggleSelectionItalic()
* \memberof clove::RichEditBox
* \brief Toggles the italic-flag for the current selection.
*/        

/*!
* \fn toggleSelectionUnderline()
* \memberof clove::RichEditBox
* \brief Toggles the underline-flag for the current selection.
*/        

/*!
* \fn selectionIncreaseFontSize()
* \memberof clove::RichEditBox
* \brief Increases the font size for the current selection.
*/        

/*!
* \fn selectionDecreaseFontSize()
* \memberof clove::RichEditBox
* \brief Decreases the font size for the current selection.
*/        

/*!
* \fn selectionSetForegroundColor()
* \memberof clove::RichEditBox
* \brief Sets the foreground color for the current selection.
* \param c The color string.
*/        

/*!
* \fn selectionSetBackgroundColor()
* \memberof clove::RichEditBox
* \brief Sets the background color for the current selection.
* \param c The color string.
*/        

/*!
* \fn selectionInsertList()
* \memberof clove::RichEditBox
* \brief Inserts a list at the current place.
* 
* config may contain:
* - `ordered`: If the list is ordered.
* \param config The list configuration.
*/        

/*!
* \class clove::RichEdit
* \extends clove::Widget
* \brief A user input box for text with rich formatting.
* 
* This widget includes formatting toolbars.
* For a bare scriptable edit box, see clove::RichEditBox.
*/        

/*!
* \fn htmlContent()
* \memberof clove::RichEdit
* \brief The content text as html string.
*/        

/*!
* \fn setHtmlContent()
* \memberof clove::RichEdit
* \brief Setter for htmlContent().
* \param value The new value.
*/        

/*!
* \fn richeditbox()
* \memberof clove::RichEdit
* \brief Returns the inner clove::RichEditBox.
*/        

/*!
* \class clove::Slider
* \extends clove::Widget
* \brief A slider allows the user to choose a number from a given value interval.
* 
* The user chooses a value by moving a handle alongside a line.
*/        

/*!
* \fn min()
* \memberof clove::Slider
* \brief The minimum value.
*/        

/*!
* \fn setMin()
* \memberof clove::Slider
* \brief Setter for min().
* \param value The new value.
*/        

/*!
* \fn max()
* \memberof clove::Slider
* \brief The maximum value.
*/        

/*!
* \fn setMax()
* \memberof clove::Slider
* \brief Setter for max().
* \param value The new value.
*/        

/*!
* \fn stepsize()
* \memberof clove::Slider
* \brief The step size.
*/        

/*!
* \fn setStepsize()
* \memberof clove::Slider
* \brief Setter for stepsize().
* \param value The new value.
*/        

/*!
* \fn value()
* \memberof clove::Slider
* \brief The current slider value.
*/        

/*!
* \fn setValue()
* \memberof clove::Slider
* \brief Setter for value().
* \param value The new value.
*/        

/*!
* \fn orientation()
* \memberof clove::Slider
* \brief If to present the slider `'vertical'`ly or `'horizontal'`ly.
*/        

/*!
* \fn setOrientation()
* \memberof clove::Slider
* \brief Setter for orientation().
* \param value The new value.
*/        

/*!
* \property OnChanged
* \memberof clove::Slider
* \brief Triggered when the slider value changed.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \class clove::Expander
* \extends clove::Widget
* \brief A single-widget container which can be expanded or collapsed.
*/   

/*!
* \fn isExpanded()
* \memberof clove::Expander
* \brief If the expander is expanded.
*/        

/*!
* \fn setIsExpanded()
* \memberof clove::Expander
* \brief Setter for isExpanded().
* \param value The new value.
*/     

/*!
* \fn collapsedIcon()
* \memberof clove::Expander
* \brief The icon for the expander when collapsed.
*/        

/*!
* \fn setCollapsedIcon()
* \memberof clove::Expander
* \brief Setter for collapsedIcon().
* \param value The new value.
*/     

/*!
* \fn collapsedLabel()
* \memberof clove::Expander
* \brief The label for the expander when collapsed.
*/        

/*!
* \fn setCollapsedLabel()
* \memberof clove::Expander
* \brief Setter for collapsedLabel().
* \param value The new value.
*/     

/*!
* \fn expandedIcon()
* \memberof clove::Expander
* \brief The icon for the expander when expanded.
*/        

/*!
* \fn setExpandedIcon()
* \memberof clove::Expander
* \brief Setter for expandedIcon().
* \param value The new value.
*/     

/*!
* \fn expandedLabel()
* \memberof clove::Expander
* \brief The label for the expander when expanded.
*/        

/*!
* \fn setExpandedLabel()
* \memberof clove::Expander
* \brief Setter for expandedLabel().
* \param value The new value.
*/     

/*!
* \fn body()
* \memberof clove::Expander
* \brief The inner clove::Widget.
*/        

/*!
* \fn setBody()
* \memberof clove::Expander
* \brief Setter for body().
* \param value The new value.
*/     

/*!
* \class clove::RawResizeSplitter
* \extends clove::Widget
* \brief The actual splitter in a clove::ResizeSplitter.
* 
* Not intended for direct usage in a user interface!
*/        

/*!
* \property OnMoveBegin
* \memberof clove::RawResizeSplitter
* \brief Triggered when the user begins to move the splitter.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnMove
* \memberof clove::RawResizeSplitter
* \brief Triggered subsequently while the user moves the splitter.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnMoveEnd
* \memberof clove::RawResizeSplitter
* \brief Triggered when the user ends moving the splitter.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \class clove::ResizeSplitter
* \extends clove::Widget
* \brief A container for two widgets with a splitter for manual resizing between them.
*/        

/*!
* \fn orientation()
* \memberof clove::ResizeSplitter
* \brief If the two widgets are stacked `'vertical'`ly or `'horizontal'`ly.
*/        

/*!
* \fn setOrientation()
* \memberof clove::ResizeSplitter
* \brief Setter for orientation().
* \param value The new value.
*/        

/*!
* \fn widget1()
* \memberof clove::ResizeSplitter
* \brief The first widget as widget configuration like for clove::build().
*/        

/*!
* \fn setWidget1()
* \memberof clove::ResizeSplitter
* \brief Setter for widget1().
* \param value The new value.
*/        

/*!
* \fn widget2()
* \memberof clove::ResizeSplitter
* \brief The second widget as widget configuration like for clove::build().
*/        

/*!
* \fn setWidget2()
* \memberof clove::ResizeSplitter
* \brief Setter for widget2().
* \param value The new value.
*/        

/*!
* \fn position()
* \memberof clove::ResizeSplitter
* \brief The current splitter position as value between 0.0 and 1.0.
*/        

/*!
* \fn setPosition()
* \memberof clove::ResizeSplitter
* \brief Setter for position().
* \param value The new value.
*/        

/*!
* \fn showOnly()
* \memberof clove::ResizeSplitter
* \brief Specifies if to show just one of the two sides (`0`, `1`, `undefined`).
*/        

/*!
* \fn setShowOnly()
* \memberof clove::ResizeSplitter
* \brief Setter for showOnly().
* \param value The new value.
*/        

/*!
* \fn splitterWidth()
* \memberof clove::ResizeSplitter
* \brief The splitter width (height for vertical orientation) as css length.
*/        

/*!
* \fn setSplitterWidth()
* \memberof clove::ResizeSplitter
* \brief Setter for splitterWidth().
* \param value The new value.
*/        

/*!
* \fn splitterVisibility()
* \memberof clove::ResizeSplitter
* \brief The visibility of the splitter. See clove::Widget::visibility() for details.
*/        

/*!
* \fn setSplitterVisibility()
* \memberof clove::ResizeSplitter
* \brief Setter for splitterVisibility().
* \param value The new value.
*/        

/*!
* \class clove::Border
* \extends clove::Widget
* \brief A single-widget container framing the inner one with (a css-styled) border.
*/        

/*!
* \fn body()
* \memberof clove::Border
* \brief The inner widget as widget configuration like for clove::build().
*/        

/*!
* \fn setBody()
* \memberof clove::Border
* \brief Setter for body().
* \param value The new value.
*/     

/*!
* \class clove::Form
* \extends clove::Widget
* \brief A container for a form-like with a label for each entry in a row-oriented alignment.
*/        

/*!
* \fn sections()
* \memberof clove::Form
* \brief A list of widget configurations as for clove::build(). Each defines one section in the form.
* 
* Define the labels in the `formSectionLabel` property of each widget.
*/        

/*!
* \fn setSections()
* \memberof clove::Form
* \brief Setter for sections().
* \param value The new value.
*/     

/*!
* \class clove::ProgressBar
* \extends clove::Widget
* \brief A progress bar.
*/        

/*!
* \fn value()
* \memberof clove::ProgressBar
* \brief The progress value between `0.0` and `1.0`.
*/        

/*!
* \fn setValue()
* \memberof clove::ProgressBar
* \brief Setter for value().
* \param value The new value.
*/     

/*!
* \fn orientation()
* \memberof clove::ProgressBar
* \brief If to present the progress bar `'vertical'`ly or `'horizontal'`ly.
*/        

/*!
* \fn setOrientation()
* \memberof clove::ProgressBar
* \brief Setter for orientation().
* \param value The new value.
*/     

/*!
* \fn label()
* \memberof clove::ProgressBar
* \brief An optional additional label text, which is displayed combined with the progress value.
*/        

/*!
* \fn setLabel()
* \memberof clove::ProgressBar
* \brief Setter for label().
* \param value The new value.
*/     

/*!
* \fn labelFunction()
* \memberof clove::ProgressBar
* \brief A `function(value, widget)` which returns a custom label text.
*/        

/*!
* \fn setLabelFunction()
* \memberof clove::ProgressBar
* \brief Setter for labelFunction().
* \param value The new value.
*/     

/*!
* \class clove::NotificationController
* \brief Controller for clove notifications.
* 
* Notifications are small popups, typically in the top/right corner of the window. They can inform the user about some status changes or provide some ways for user interactions.
* 
* Use clove::notifications.
*/ 

/*!
* \fn notify()
* \memberof clove::NotificationController
* \brief Opens a new notification.
* 
* notificationConfig may contain:
* - `timeout`: Closes the notification automatically after this amount of seconds.
* - `closeable`: If to provide a close button.
* - `priority`: Controls stuff like ordering. Default is 0, higher values mean more importance.
* \param config A widget configuration for the notification body, as for clove::build().
* \param notificationConfig Some configuration aspects about the notification.
* \returns The clove::Widget implementing the notification. Use it for operations like removal or getting subwidget. This widget spans a new clove::NameScope!
*/     

/*!
* \property notifications
* \memberof clove
* \brief The clove notification controller.
* 
* It is an instance of clove::NotificationController.
*/        

/*!
* \class clove::MediaPlayer
* \extends clove::Widget
* \brief A base class for media player widgets.
* 
* This is an abstract class. Use one of the subclasses.
* 
* Note: This interface provides actions and events for typical usage.
* For other scenarios, use nativeNode() for retrieving the `<audio>` or
* `<video>` html dom node. They provide a more powerful api.
*/

/*!
* \fn autoPlay()
* \memberof clove::MediaPlayer
* \brief If to automatically start playback as soon as possible.
*/        

/*!
* \fn setAutoPlay()
* \memberof clove::MediaPlayer
* \brief Setter for autoPlay().
* \param value The new value.
*/     

/*!
* \fn showControls()
* \memberof clove::MediaPlayer
* \brief If to show user controls for play, pause and more.
*/        

/*!
* \fn setShowControls()
* \memberof clove::MediaPlayer
* \brief Setter for showControls().
* \param value The new value.
*/     

/*!
* \fn currentMediaPosition()
* \memberof clove::MediaPlayer
* \brief The current media playback position in seconds.
*/        

/*!
* \fn setCurrentMediaPosition()
* \memberof clove::MediaPlayer
* \brief Setter for currentMediaPosition().
* \param value The new value.
*/     

/*!
* \fn loop()
* \memberof clove::MediaPlayer
* \brief If to playback in an endless loop.
*/        

/*!
* \fn setLoop()
* \memberof clove::MediaPlayer
* \brief Setter for loop().
* \param value The new value.
*/     

/*!
* \fn muted()
* \memberof clove::MediaPlayer
* \brief If to mute the audio playback.
*/        

/*!
* \fn setMuted()
* \memberof clove::MediaPlayer
* \brief Setter for muted().
* \param value The new value.
*/     

/*!
* \fn preload()
* \memberof clove::MediaPlayer
* \brief If to begin loading the media data as soon as possible.
*/        

/*!
* \fn setPreload()
* \memberof clove::MediaPlayer
* \brief Setter for preload().
* \param value The new value.
*/     

/*!
* \fn volume()
* \memberof clove::MediaPlayer
* \brief The audio playback volume between `0.0` and `1.0`.
*/        

/*!
* \fn setVolume()
* \memberof clove::MediaPlayer
* \brief Setter for volume().
* \param value The new value.
*/     

/*!
* \fn source()
* \memberof clove::MediaPlayer
* \brief The media source URL.
*/        

/*!
* \fn setSource()
* \memberof clove::MediaPlayer
* \brief Setter for source().
* \param value The new value.
*/     

/*!
* \fn duration()
* \memberof clove::MediaPlayer
* \brief The duration of the loaded media source in seconds.
*/        

/*!
* \fn hasEnded()
* \memberof clove::MediaPlayer
* \brief If the playback has ended (i.e. reached the end).
*/        

/*!
* \fn isPaused()
* \memberof clove::MediaPlayer
* \brief If the playback is logically paused.
* 
* This does not return `true` just when loading stalls.
*/        

/*!
* \fn nativeNode()
* \memberof clove::MediaPlayer
* \brief Returns the native html dom node.
*/        

/*!
* \fn play()
* \memberof clove::MediaPlayer
* \brief Starts playback.
*/        

/*!
* \fn pause()
* \memberof clove::MediaPlayer
* \brief Pauses playback.
*/        

/*!
* \property OnLoadingAborted
* \memberof clove::MediaPlayer
* \brief Triggered when the media loading aborted for some reasons (e.g. network errors).
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnReadyForPlayback
* \memberof clove::MediaPlayer
* \brief Triggered when there are enough data for starting playback.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnDurationChanged
* \memberof clove::MediaPlayer
* \brief Triggered when the playback duration changed.
* 
* See also duration().
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnHasEnded
* \memberof clove::MediaPlayer
* \brief Triggered when the playback has ended.
* 
* See also hasEnded().
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnIsPaused
* \memberof clove::MediaPlayer
* \brief Triggered when the playback logically paused.
* 
* This is not triggered when loading stalls.
* 
* This is a clove.Event instance. See Manual for details.
*/        

/*!
* \property OnIsPlaying
* \memberof clove::MediaPlayer
* \brief Triggered when the playback logically starts.
* 
* This is not triggered when loading has stalled and resumes.
* 
* This is a clove.Event instance. See Manual for details.
*/        
    
/*!
* \class clove::AudioPlayer
* \extends clove::MediaPlayer
* \brief A widget which plays an audio source and optionally allows user controls.
*/        

/*!
* \fn static canPlayType()
* \memberof clove::AudioPlayer
* \brief Determines if the browser can play a certain audio type and returns a string as described in the html specs (->"canPlayType").
* \param t A mimetype string.
*/     
    
/*!
* \class clove::VideoPlayer
* \extends clove::MediaPlayer
* \brief A widget which plays a video source and optionally allows user controls.
*/        

/*!
* \fn static canPlayType()
* \memberof clove::VideoPlayer
* \brief Determines if the browser can play a certain video type and returns a string as described in the html specs (->"canPlayType").
* \param t A mimetype string.
*/     
