# Copyright (C) 2016-2017, Josef Hahn
#
# This file is part of Clove.
#
# Clove is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Clove is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Clove.  If not, see <http://www.gnu.org/licenses/>.

import http.server
import socketserver
import urllib.parse
import errno
import time
import threading
import sys
import subprocess
import os
import cgi


running = True
passed = None
details = None

def startserver():
    port = 8309
    connected = False
    while not connected:
        try:
            httpd = socketserver.ThreadingTCPServer(("", port), WebconsoleHTTPRequestHandler)
            connected = True
        except OSError as e:
            if e.errno == errno.EADDRINUSE:
                port += 1
            else:
                raise
    url = "http://localhost:{}/".format(port)
    threading.Thread(target=httpd.serve_forever, daemon=True).start()
    return url


class WebconsoleHTTPRequestHandler(http.server.SimpleHTTPRequestHandler):

    def log_message(self, format, *args):
        pass

    def do_HEAD(self):
        self.send_response(200)
        self.send_header("Content-type", "text/html")
        self.end_headers()

    def do_POST(self):
        global running
        global passed
        global details
        form = cgi.FieldStorage(
            fp=self.rfile, 
            headers=self.headers,
            environ={'REQUEST_METHOD':'POST', 'CONTENT_TYPE':self.headers['Content-Type'],
        })
        self.send_response(200)
        self.send_header("Content-type", "text/html;charset=utf-8")
        self.end_headers()
        running = False
        passed = form["status"].value == "passed"
        details = form["details"].value if ("details" in form) else None


testrootdir = os.path.dirname(os.path.abspath(__file__))

if sys.argv[1] == "list":
    if len(sys.argv) >= 3:
        testgroupdir = testrootdir + "/" + sys.argv[2]
        for x in os.listdir(testgroupdir):
            if x.endswith(".html"):
                print(x)
    else:
        for x in os.listdir(testrootdir):
            if os.path.isdir(testrootdir + "/" + x):
                print(x)
    sys.exit(0)

test = sys.argv[1]
browser = sys.argv[2]
callbackurl = startserver()
testurl = "file:///" + testrootdir.replace("\\", "/") + "/" + test.replace("\\", "/")

subprocess.call([browser, testurl+"#"+urllib.parse.quote_plus(callbackurl)])

while running:
    time.sleep(0.1)
    
if details is not None:
    print(details)

if not passed:
    sys.exit(1)
sys.exit(0)