/*
 @licstart  The following is the entire license notice for the JavaScript code in this page.

 Copyright (C) 2016-2017, Josef Hahn

 This file is part of Clove.

 Clove is free software: you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation, either version 3 of the License, or
 (at your option) any later version.

 Clove is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with Clove.  If not, see <http://www.gnu.org/licenses/>.

 @licend  The above is the entire license notice for the JavaScript code in this page.
*/

var _ct_testdone = false;
var _ct_bdiv;
var _ct_tdiv;
var _ct_ddiv;

function _notifytestresult(data) {
    var h = window.location.href;
    var i = h.lastIndexOf("#");
    if (i > -1) {
        var callbackaddress = decodeURIComponent(h.substring(i+1));
        $.ajax({
            url: callbackaddress,
            data: data,
            method: 'POST',
        });
    }
}

function _testpassed() {
    if (_ct_testdone)
        return;
    _ct_testdone = true;
    _ct_tdiv.text("Test completed!");
    _ct_tdiv.css("color", "#060");
    _notifytestresult({status: 'passed'});
}

function _testfailed(details) {
    if (_ct_testdone)
        return;
    _ct_testdone = true;
    _ct_tdiv.text("Test failed!");
    _ct_tdiv.css("color", "#900");
    _ct_ddiv.text(details);
    _notifytestresult({status: 'failed', details: details});
}

function _runtest(fcts) {
    if (fcts.length > 0) {
        var fct = fcts[0];
        try {
            fct();
        }
        catch(e) {
            _testfailed(e);
            return;
        }
        fcts.splice(0, 1);
    }
    if (fcts.length > 0) {
        setTimeout(function(){_runtest(fcts);}, 10);
    }
    else
        _testpassed();
}

function runtest(fcts) {
    $(document).ready(function(){
        _ct_bdiv = $("<div style='position:absolute; top:0; left:0; right:0; bottom:0; opacity:0.5; background:#ccddff; z-index:1;'></div>");
        _ct_tdiv = $("<div style='position:absolute; top:0; left:0; right:0; bottom:50%; z-index:1; padding: 50pt; font-size: 2em; font-weight: 100; font-family: sans-serif;'>Running tests...</div>");
        _ct_ddiv = $("<div style='position:absolute; top:50%; left:0; right:0; bottom:0; z-index:1; padding: 5pt; color: #555; overflow: auto; white-space: pre-wrap;'></div>");
        $(document.body).append(_ct_bdiv);
        $(document.body).append(_ct_tdiv);
        $(document.body).append(_ct_ddiv);
        window.onerror = function(msg, url, line, col, error) {
            _testfailed(error || (url+":"+line+"\n"+msg));
        };
        _runtest(fcts);
    });
}

function _assertionfailed(asserttext, text) {
    throw new Error("Assertion failed: " + asserttext + "\n" + (text||""));
}

function assertEqual(a, b, text) {
    if (a !== b)
        _assertionfailed(a+"=="+b, text);
}

function assertNotUndefined(a, text) {
    if (a === undefined)
        _assertionfailed("not undefined", text);
}

function assertPosition(elem, x, y, cfg, text) {
    if (elem instanceof clove.Widget)
        elem = elem.topnode;
    cfg = clove.utils.applyDefaultsToConfig(cfg, {
        relativeTo: document.body,
    });
    var p = $(elem).position(cfg.relativeTo);
    var xdiff = Math.abs(p.left - x);
    var ydiff = Math.abs(p.top - y);
    if (xdiff > 4 || ydiff > 4)
        _assertionfailed("position "+x+","+y, text);
}
